import * as cheerio from 'cheerio';
import * as puppeteer from 'puppeteer';
import fs from 'fs';

import { allTab, baseUrl, exampleTab, taskName } from './constants.ts';
import { getCrawlConfig, setCrawlConfig } from './constants.ts';
// Import the errors object from puppeteer

const crawlConfig = getCrawlConfig();
export const sleep = (ms: number) => new Promise((resolve) => setTimeout(resolve, ms));
let errorCount = 5;
let browser: puppeteer.Browser;
let currentTabIndex = crawlConfig.currentTabIndex || 0;

const initializeBrowser = async () => {
  if (!browser) {
    browser = await puppeteer.launch({
      headless: true, // 继续使用无头模式
      args: [
        '--no-sandbox',
        '--disable-setuid-sandbox',
        '--disable-blink-features=AutomationControlled', // 禁用自动化控制标志
      ],
      ignoreDefaultArgs: ['--enable-automation'],
      devtools: false,
      defaultViewport: {
        width: 1280,
        height: 720,
      },
    });
  }
};

const closeBrowser = async () => {
  if (browser) {
    await browser.close();
  }
};

const getPageContent = async (link: string, retryCount = 3) => {
  if (!browser) {
    await initializeBrowser();
  }
  let page = await browser.newPage();
  // 设置用户代理和语言，模拟常规浏览器
  await page.setUserAgent('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36');
  await page.setExtraHTTPHeaders({
    'Accept-Language': 'en-US,en;q=0.9',
  });
  console.log('start crawl tab', currentTabIndex, link);
  try {
    await page.goto(link, { timeout: 120000 }); // 将超时时间增加到120秒
  } catch (error) {
    if (error instanceof puppeteer.TimeoutError) {
      console.log('Timeout error when accessing:', link);
      await page.close();
      if (retryCount > 0) {
        console.log('Retrying...', retryCount, 'attempts left');
        await sleep(10000); // 等待10秒
        await browser.close();
        browser = null;
        await sleep(10000); // 等待10秒
        return await getPageContent(link, retryCount - 1); // Retry with decremented retryCount
      } else {
        console.log('Skipping after 3 retries:', link);
        return null; // Return null or handle as needed when retries are exhausted
      }
    } else {
      console.log('Non-timeout error:', error.message);
      await page.close();
      throw error; // Re-throw non-timeout errors to handle them in the calling function
    }
  }
  console.log('end crawl tab', currentTabIndex, link);
  const html = await page.content();
  await page.close();
  return html;
};

export const crawlTab = async (link: string, deep = true): Promise<LinkTitleTimeCrawl[]> => {
  try {
    const html = await getPageContent(link);
    const $ = cheerio.load(html);
    // class为style01 mB10的div
    const title = $('.style01.mB10').text();
    // 获取id为contentDiv的所有的文本内容
    const contentDiv = $('#contentDiv');
    // contentDiv 下面的class为newsText的内容
    const content = contentDiv.find('.newsText').text();
    // class为pagenavbar的是分页导航
    const pageNavbar = $('.pagenavbar');
    const aList = pageNavbar.find('a');
    const aListArray = aList.toArray();
    let aListFullLink = aListArray
      .map((item) => {
        const a = $(item);
        const href = a.attr('href');
        return getFullLink(href);
      })
      .filter((item) => {
        // 过滤 ?start=0 和 当前的链接
        return !item.includes('?start=0') && item !== link;
      });
    // 去重复
    let page = [];
    if (deep && aListFullLink.length > 0) {
      aListFullLink = [...new Set(aListFullLink)];
      console.log('aListFullLink', aListFullLink);
      for (const item of aListFullLink) {
        const one = await crawlTab(item, false);
        page.push(...one);
      }
      page = page.flat();
      return [...page, { title, content, link: link, deep: deep }];
    }
    errorCount = 0;
    return [{ title, content, link: link, deep: deep }, ...page];
  } catch (e) {
    // 如果是超时错误
    if (e instanceof puppeteer.TimeoutError) {
      console.error('crawlTab error', 'currentTabIndex', currentTabIndex, 'link', link, 'error', e.message);
      await closeBrowser();
      console.error('close browser', 'errorCount', errorCount);
      // 重试
      errorCount++;
      if (errorCount > 3) {
        console.error('crawlTab error and return empty', 'currentTabIndex', currentTabIndex, link);
        return [];
      }
      await sleep(3000);
      return await crawlTab(link, deep);
    } else {
      console.error('crawlTab other error', 'currentTabIndex', currentTabIndex, 'link', link, 'error', e.message);
      return [];
    }
  }
};
export const getFullLink = (link: string) => {
  return `${baseUrl}${link}`;
};
type LinkTitleTime = {
  link: string;
  fullLink?: string;
  title: string;
  time: string;
  crawlTime: string;
  crawl?: { title: string; content: string; link: string; page: string; deep: boolean }[];
};
type LinkTitleTimeCrawl = { title: string; content: string; link: string; page: string; deep: boolean };
export const crawlTabList = async (link: string, fileName?: string) => {
  await initializeBrowser();

  const html = await getPageContent(link);
  const $ = cheerio.load(html);
  // 获取id为contentDiv的所有的文本内容
  const title = $('h1').text();
  const newsContent02 = $('.newsContent02');
  const linkTitleTimeList: LinkTitleTime[] = [];
  const nowTime = new Date().getTime();
  newsContent02.find('dt').each((index, element) => {
    const link = $(element).find('a').attr('href');
    const title = $(element).find('a').text();
    let time = $(element).find('span').text();
    console.log('Link:', link);
    console.log('Title:', title);
    console.log('Time:', time);
    // time包含的时候，去掉(和)
    time = time.replace(/\(/, '');
    time = time.replace(/\)/, '');
    const createTime = new Date(time).getTime();
    // 当前的时间和createTime的差值，有效期在二周内
    const diffTime = nowTime - createTime;
    if (diffTime > 14 * 24 * 60 * 60 * 1000) {
      console.log('time is out of date', time);
      return;
    }
    linkTitleTimeList.push({ link, title, fullLink: getFullLink(link), time, crawlTime: new Date().toISOString() });
  });

  // console.log('linkTitleTimeList', linkTitleTimeList, linkTitleTimeList.length);
  linkTitleTimeList.length && console.log('current linkTitleTimeList', linkTitleTimeList.length);
  for (const item of linkTitleTimeList) {
    const fullLink = getFullLink(item.link);
    //
    item.crawl = await crawlTab(fullLink, true);
  }
  fileName = fileName || 'a.json';
  const filePath = `./data/${taskName}/${fileName}`;
  fs.writeFileSync(filePath, JSON.stringify(linkTitleTimeList, null, 2));
  return title;
};

const tabExampleUrl = `${baseUrl}${exampleTab.href}`;

const startTime = new Date().getTime();
const crawlAllTab = async () => {
  await initializeBrowser();
  console.time(`${taskName} crawlAllTab`);
  for (let i = currentTabIndex; i < allTab.length; i++) {
    const tab = allTab[i];
    const link = `${baseUrl}${tab.href}`;
    console.log('crawl tab', tab.parentName, tab.text, link);
    try {
      await crawlTabList(link, `${currentTabIndex}.json`);
    } catch (error) {
      console.log('crawl tab error', link, error);
    }
    const diffTime = (new Date().getTime() - startTime) / 1000;
    console.log('crawl tab end', tab.parentName, tab.text, link, 'run-time:', diffTime, 's\n\n');
    currentTabIndex++;
    setCrawlConfig({
      currentTabIndex,
    });
  }
  await closeBrowser();
  setCrawlConfig({
    currentTabIndex: 0,
  });
  console.timeEnd(`${taskName} crawlAllTab`);
};

crawlAllTab();
// crawlTabList(tabExampleUrl, 'a.json');
